@@ -73,6 +73,9 @@ extern "C" {
 
 #ifndef MRN_MARIADB_P
 #  define MRN_HANDLER_HAVE_INDEX_READ_LAST_MAP
+#  if MYSQL_VERSION_ID >= 50611
+#    define MRN_HANDLER_HAVE_HA_INDEX_READ_LAST_MAP
+#  endif
 #endif
 
 #if (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 50302)
@@ -217,6 +220,7 @@ private:
   THR_LOCK_DATA thr_lock_data;
 
   // for wrapper mode (TODO: need to be confirmed)
+  uint      wrap_ft_init_count;
   MRN_SHARE *share;
   KEY       *wrap_key_info;
   KEY       *base_key_info;
@@ -252,6 +256,7 @@ private:
   grn_ctx *ctx;
   grn_obj *grn_table;
   grn_obj **grn_columns;
+  grn_obj **grn_column_ranges;
   grn_obj **grn_index_tables;
   grn_obj **grn_index_columns;
 
@@ -540,7 +545,7 @@ private:
   void remove_related_files(const char *base_path);
   void remove_grn_obj_force(const char *name);
   grn_obj *find_tokenizer(const char *name, int name_length);
-  grn_obj *find_normalizer(Field *field);
+  grn_obj *find_normalizer(KEY *key_info);
   int wrapper_get_next_record(uchar *buf);
   int storage_get_next_record(uchar *buf);
   void geo_store_rectangle(const uchar *rectangle);
@@ -551,7 +556,7 @@ private:
 #endif
   bool is_dry_write();
   bool is_enable_optimization();
-  bool is_need_normalize(Field *field) const;
+  bool should_normalize(Field *field) const;
   bool is_temporary_table_name(const char *name) const;
   void check_count_skip(key_part_map start_key_part_map,
                         key_part_map end_key_part_map, bool fulltext);
@@ -654,8 +659,7 @@ private:
   int storage_encode_key(Field *field, const uchar *key, uchar *buf, uint *size);
   int storage_encode_multiple_column_key(KEY *key_info,
                                          const uchar *key, uint key_length,
-                                         uchar *buffer, uint *encoded_length,
-                                         bool decode);
+                                         uchar *buffer, uint *encoded_length);
   int storage_encode_multiple_column_key_range(KEY *key_info,
                                                const key_range *start,
                                                const key_range *end,
@@ -692,6 +696,10 @@ private:
                                   Field *field, grn_obj *table_obj, int &error);
 #endif
   int storage_create_validate_index(TABLE *table);
+  int storage_create_index_table(TABLE *table, const char *grn_table_name,
+                                 grn_obj *grn_table, MRN_SHARE *tmp_share,
+                                 KEY *key_info, grn_obj **index_tables,
+                                 uint i);
   int storage_create_index(TABLE *table, const char *grn_table_name,
                            grn_obj *grn_table, MRN_SHARE *tmp_share,
                            KEY *key_info, grn_obj **index_tables,
