@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2011 Kentoku SHIBA
-  Copyright(C) 2011 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2011-2013 Kentoku SHIBA
+  Copyright(C) 2011-2013 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <string>
+
 #include "mrn_mysql.h"
 
 #if MYSQL_VERSION_ID >= 50500
@@ -55,22 +57,11 @@ extern handlerton *mrn_hton_ptr;
 extern HASH mrn_allocated_thds;
 extern pthread_mutex_t mrn_allocated_thds_mutex;
 
-char *mrn_create_string(const char *str, uint length)
+static char *mrn_get_string_between_quote(const char *ptr)
 {
-  char *res;
-  DBUG_ENTER("mrn_create_string");
-  if (!(res = (char*) my_malloc(length + 1, MYF(MY_WME))))
-    DBUG_RETURN(NULL);
-  memcpy(res, str, length);
-  res[length] = '\0';
-  DBUG_RETURN(res);
-}
-
-char *mrn_get_string_between_quote(char *ptr, bool alloc)
-{
-  char *start_ptr, *end_ptr, *tmp_ptr, *esc_ptr;
+  const char *start_ptr, *end_ptr, *tmp_ptr, *esc_ptr;
   bool find_flg = FALSE, esc_flg = FALSE;
-  DBUG_ENTER("mrn_get_string_between_quote");
+  MRN_DBUG_ENTER_FUNCTION();
 
   start_ptr = strchr(ptr, '\'');
   end_ptr = strchr(ptr, '"');
@@ -126,47 +117,49 @@ char *mrn_get_string_between_quote(char *ptr, bool alloc)
   } else
     DBUG_RETURN(NULL);
 
-  *end_ptr = '\0';
-  if (esc_flg)
-  {
-    esc_ptr = start_ptr;
-    while (TRUE)
-    {
-      esc_ptr = strchr(esc_ptr, '\\');
-      if (!esc_ptr)
-        break;
-      switch(*(esc_ptr + 1))
-      {
-        case 'b':
-          *esc_ptr = '\b';
-          break;
-        case 'n':
-          *esc_ptr = '\n';
-          break;
-        case 'r':
-          *esc_ptr = '\r';
-          break;
-        case 't':
-          *esc_ptr = '\t';
-          break;
-        default:
-          *esc_ptr = *(esc_ptr + 1);
-          break;
+  size_t length = end_ptr - start_ptr;
+  char *extracted_string = (char *)my_malloc(length + 1, MYF(MY_WME));
+  if (esc_flg) {
+    size_t extracted_index = 0;
+    const char *current_ptr = start_ptr;
+    while (current_ptr < end_ptr) {
+      if (*current_ptr != '\\') {
+        extracted_string[extracted_index] = *current_ptr;
+        ++extracted_index;
+        current_ptr++;
+        continue;
       }
-      esc_ptr++;
-      strcpy(esc_ptr, esc_ptr + 1);
+
+      if (current_ptr + 1 == end_ptr) {
+        break;
+      }
+
+      switch (*(current_ptr + 1))
+      {
+      case 'b':
+        extracted_string[extracted_index] = '\b';
+        break;
+      case 'n':
+        extracted_string[extracted_index] = '\n';
+        break;
+      case 'r':
+        extracted_string[extracted_index] = '\r';
+        break;
+      case 't':
+        extracted_string[extracted_index] = '\t';
+        break;
+      default:
+        extracted_string[extracted_index] = *(current_ptr + 1);
+        break;
+      }
+      ++extracted_index;
     }
-  }
-  if (alloc)
-  {
-    DBUG_RETURN(
-      mrn_create_string(
-      start_ptr,
-      strlen(start_ptr))
-    );
   } else {
-    DBUG_RETURN(start_ptr);
+    memcpy(extracted_string, start_ptr, length);
+    extracted_string[length] = '\0';
   }
+
+  DBUG_RETURN(extracted_string);
 }
 
 #ifdef WITH_PARTITION_STORAGE_ENGINE
@@ -178,7 +171,7 @@ void mrn_get_partition_info(const char *table_name, uint table_name_length,
   partition_info *part_info = table->part_info;
   partition_element *tmp_part_elem = NULL, *tmp_sub_elem = NULL;
   bool tmp_flg = FALSE, tmp_find_flg = FALSE;
-  DBUG_ENTER("mrn_get_partition_info");
+  MRN_DBUG_ENTER_FUNCTION();
   *part_elem = NULL;
   *sub_elem = NULL;
   if (!part_info)
@@ -254,7 +247,7 @@ void mrn_get_partition_info(const char *table_name, uint table_name_length,
     if (!share->param_name) \
     { \
       if ((share->param_name = mrn_get_string_between_quote( \
-        start_ptr, TRUE))) \
+        start_ptr))) \
         share->MRN_PARAM_STR_LEN(param_name) = strlen(share->param_name); \
       else { \
         error = ER_MRN_INVALID_TABLE_PARAM_NUM; \
@@ -274,7 +267,7 @@ void mrn_get_partition_info(const char *table_name, uint table_name_length,
     if (!share->param_name[param_pos]) \
     { \
       if ((share->param_name[param_pos] = mrn_get_string_between_quote( \
-        start_ptr, TRUE))) \
+        start_ptr))) \
         share->MRN_PARAM_STR_LEN(param_name)[param_pos] = \
           strlen(share->param_name[param_pos]); \
       else { \
@@ -293,14 +286,13 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
 {
   int i, error;
   int title_length;
-  char *param_string = NULL;
-  char *sprit_ptr[2];
-  char *tmp_ptr, *start_ptr;
+  const char *sprit_ptr[2];
+  const char *tmp_ptr, *start_ptr;
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   partition_element *part_elem;
   partition_element *sub_elem;
 #endif
-  DBUG_ENTER("mrn_parse_table_param");
+  MRN_DBUG_ENTER_FUNCTION();
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   mrn_get_partition_info(share->table_name, share->table_name_length, table,
     &part_elem, &sub_elem);
@@ -311,11 +303,8 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
   for (i = 2; i > 0; i--)
 #endif
   {
-    if (param_string)
-    {
-      my_free(param_string, MYF(0));
-      param_string = NULL;
-    }
+    const char *params_string_value;
+    uint params_string_length;
     switch (i)
     {
 #ifdef WITH_PARTITION_STORAGE_ENGINE
@@ -323,106 +312,100 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
         if (!sub_elem || !sub_elem->part_comment)
           continue;
         DBUG_PRINT("info", ("mroonga create sub comment string"));
-        if (
-          !(param_string = mrn_create_string(
-            sub_elem->part_comment,
-            strlen(sub_elem->part_comment)))
-        ) {
-          error = HA_ERR_OUT_OF_MEM;
-          goto error_alloc_param_string;
-        }
-        DBUG_PRINT("info", ("mroonga sub comment string=%s", param_string));
+        params_string_value = sub_elem->part_comment;
+        params_string_length = strlen(params_string_value);
+        DBUG_PRINT("info",
+                   ("mroonga sub comment string=%s", params_string_value));
         break;
       case 3:
         if (!part_elem || !part_elem->part_comment)
           continue;
         DBUG_PRINT("info", ("mroonga create part comment string"));
-        if (
-          !(param_string = mrn_create_string(
-            part_elem->part_comment,
-            strlen(part_elem->part_comment)))
-        ) {
-          error = HA_ERR_OUT_OF_MEM;
-          goto error_alloc_param_string;
-        }
-        DBUG_PRINT("info", ("mroonga part comment string=%s", param_string));
+        params_string_value = part_elem->part_comment;
+        params_string_length = strlen(params_string_value);
+        DBUG_PRINT("info",
+                   ("mroonga part comment string=%s", params_string_value));
         break;
 #endif
       case 2:
         if (LEX_STRING_IS_EMPTY(table->s->comment))
           continue;
         DBUG_PRINT("info", ("mroonga create comment string"));
-        if (
-          !(param_string = mrn_create_string(
-            table->s->comment.str,
-            table->s->comment.length))
-        ) {
-          error = HA_ERR_OUT_OF_MEM;
-          goto error_alloc_param_string;
-        }
-        DBUG_PRINT("info", ("mroonga comment string=%s", param_string));
+        params_string_value = table->s->comment.str;
+        params_string_length = table->s->comment.length;
+        DBUG_PRINT("info",
+                   ("mroonga comment string=%.*s",
+                    params_string_length, params_string_value));
         break;
       default:
         if (LEX_STRING_IS_EMPTY(table->s->connect_string))
           continue;
         DBUG_PRINT("info", ("mroonga create connect_string string"));
-        if (
-          !(param_string = mrn_create_string(
-            table->s->connect_string.str,
-            table->s->connect_string.length))
-        ) {
-          error = HA_ERR_OUT_OF_MEM;
-          goto error_alloc_param_string;
-        }
-        DBUG_PRINT("info", ("mroonga connect_string=%s", param_string));
+        params_string_value = table->s->connect_string.str;
+        params_string_length = table->s->connect_string.length;
+        DBUG_PRINT("info",
+                   ("mroonga connect_string=%.*s",
+                    params_string_length, params_string_value));
         break;
     }
 
-    sprit_ptr[0] = param_string;
-    while (sprit_ptr[0])
+    if (!params_string_value) {
+      continue;
+    }
+
     {
-      if ((sprit_ptr[1] = strchr(sprit_ptr[0], ',')))
+      std::string params_string(params_string_value, params_string_length);
+      sprit_ptr[0] = params_string.c_str();
+      while (sprit_ptr[0])
       {
-        *sprit_ptr[1] = '\0';
-        sprit_ptr[1]++;
-      }
-      tmp_ptr = sprit_ptr[0];
-      sprit_ptr[0] = sprit_ptr[1];
-      while (*tmp_ptr == ' ' || *tmp_ptr == '\r' ||
-        *tmp_ptr == '\n' || *tmp_ptr == '\t')
-        tmp_ptr++;
+        if ((sprit_ptr[1] = strchr(sprit_ptr[0], ',')))
+        {
+          sprit_ptr[1]++;
+        }
+        tmp_ptr = sprit_ptr[0];
+        sprit_ptr[0] = sprit_ptr[1];
+        while (*tmp_ptr == ' ' || *tmp_ptr == '\r' ||
+               *tmp_ptr == '\n' || *tmp_ptr == '\t')
+          tmp_ptr++;
 
-      if (*tmp_ptr == '\0')
-        continue;
+        if (*tmp_ptr == '\0')
+          continue;
 
-      DBUG_PRINT("info", ("mroonga title_str=%s", tmp_ptr));
-      title_length = 0;
-      start_ptr = tmp_ptr;
-      while (*start_ptr != ' ' && *start_ptr != '\'' &&
-        *start_ptr != '"' && *start_ptr != '\0' &&
-        *start_ptr != '\r' && *start_ptr != '\n' &&
-        *start_ptr != '\t')
-      {
-        title_length++;
-        start_ptr++;
-      }
-      DBUG_PRINT("info", ("mroonga title_length=%u", title_length));
+        DBUG_PRINT("info", ("mroonga title_str=%s", tmp_ptr));
+        title_length = 0;
+        start_ptr = tmp_ptr;
+        while (*start_ptr != ' ' && *start_ptr != '\'' &&
+               *start_ptr != '"' && *start_ptr != '\0' &&
+               *start_ptr != '\r' && *start_ptr != '\n' &&
+               *start_ptr != '\t' && *start_ptr != ',')
+        {
+          title_length++;
+          start_ptr++;
+        }
+        DBUG_PRINT("info", ("mroonga title_length=%u", title_length));
 
-      switch (title_length)
-      {
+        switch (title_length)
+        {
         case 0:
           continue;
         case 6:
           MRN_PARAM_STR("engine", engine);
           error = ER_MRN_INVALID_TABLE_PARAM_NUM;
           my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-            MYF(0), tmp_ptr);
+                          MYF(0), tmp_ptr);
+          goto error;
+        case 17:
+          MRN_PARAM_STR("default_tokenizer", default_tokenizer);
+          error = ER_MRN_INVALID_TABLE_PARAM_NUM;
+          my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
+                          MYF(0), tmp_ptr);
           goto error;
         default:
           error = ER_MRN_INVALID_TABLE_PARAM_NUM;
           my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
-            MYF(0), tmp_ptr);
+                          MYF(0), tmp_ptr);
           goto error;
+        }
       }
     }
   }
@@ -431,9 +414,9 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
   {
     share->engine_length = strlen(mrn_default_wrapper_engine);
     if (
-      !(share->engine = mrn_create_string(
-        mrn_default_wrapper_engine,
-        share->engine_length))
+      !(share->engine = my_strndup(mrn_default_wrapper_engine,
+                                   share->engine_length,
+                                   MYF(MY_WME)))
     ) {
       error = HA_ERR_OUT_OF_MEM;
       goto error;
@@ -470,14 +453,9 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
     }
   }
 
-  if (param_string)
-    my_free(param_string, MYF(0));
   DBUG_RETURN(0);
 
 error:
-  if (param_string)
-    my_free(param_string, MYF(0));
-error_alloc_param_string:
   DBUG_RETURN(error);
 }
 
@@ -497,7 +475,7 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
   char *sprit_ptr[2];
   char *tmp_ptr, *start_ptr;
 #endif
-  DBUG_ENTER("mrn_add_index_param");
+  MRN_DBUG_ENTER_FUNCTION();
 
 #if MYSQL_VERSION_ID >= 50500
   if (key_info->comment.length == 0)
@@ -506,9 +484,7 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
       my_free(share->key_parser[i], MYF(0));
     }
     if (
-      !(share->key_parser[i] = mrn_create_string(
-        mrn_default_parser,
-        strlen(mrn_default_parser)))
+      !(share->key_parser[i] = my_strdup(mrn_default_parser, MYF(MY_WME)))
     ) {
       error = HA_ERR_OUT_OF_MEM;
       goto error;
@@ -518,9 +494,9 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
   }
   DBUG_PRINT("info", ("mroonga create comment string"));
   if (
-    !(param_string = mrn_create_string(
-      key_info->comment.str,
-      key_info->comment.length))
+    !(param_string = my_strndup(key_info->comment.str,
+                                key_info->comment.length,
+                                MYF(MY_WME)))
   ) {
     error = HA_ERR_OUT_OF_MEM;
     goto error_alloc_param_string;
@@ -559,12 +535,27 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
     {
       case 0:
         continue;
+      case 5:
+        MRN_PARAM_STR_LIST("table", index_table, i);
+        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
+        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
+          MYF(0), tmp_ptr);
+        goto error;
       case 6:
         MRN_PARAM_STR_LIST("parser", key_parser, i);
         error = ER_MRN_INVALID_TABLE_PARAM_NUM;
         my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
           MYF(0), tmp_ptr);
         goto error;
+      case 10:
+        if (strncasecmp("normalizer", tmp_ptr, title_length) == 0) {
+          break;
+        } else {
+          error = ER_MRN_INVALID_TABLE_PARAM_NUM;
+          my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
+            MYF(0), tmp_ptr);
+          goto error;
+        }
       default:
         error = ER_MRN_INVALID_TABLE_PARAM_NUM;
         my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
@@ -575,9 +566,7 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
 #endif
   if (!share->key_parser[i]) {
     if (
-      !(share->key_parser[i] = mrn_create_string(
-        mrn_default_parser,
-        strlen(mrn_default_parser)))
+      !(share->key_parser[i] = my_strdup(mrn_default_parser, MYF(MY_WME)))
     ) {
       error = HA_ERR_OUT_OF_MEM;
       goto error;
@@ -601,13 +590,16 @@ error_alloc_param_string:
 int mrn_parse_index_param(MRN_SHARE *share, TABLE *table)
 {
   int error;
-  DBUG_ENTER("mrn_parse_index_param");
+  MRN_DBUG_ENTER_FUNCTION();
   for (uint i = 0; i < table->s->keys; i++)
   {
     KEY *key_info = &table->s->key_info[i];
+    bool is_wrapper_mode = share->engine != NULL;
 
-    if (!(key_info->flags & HA_FULLTEXT) && !mrn_is_geo_key(key_info)) {
-      continue;
+    if (is_wrapper_mode) {
+      if (!(key_info->flags & HA_FULLTEXT) && !mrn_is_geo_key(key_info)) {
+        continue;
+      }
     }
 
     if ((error = mrn_add_index_param(share, key_info, i)))
@@ -619,29 +611,143 @@ error:
   DBUG_RETURN(error);
 }
 
+int mrn_add_column_param(MRN_SHARE *share, Field *field, int i)
+{
+  int error;
+  char *param_string = NULL;
+  int title_length;
+  char *sprit_ptr[2];
+  char *tmp_ptr, *start_ptr;
+  MRN_DBUG_ENTER_FUNCTION();
+
+  DBUG_PRINT("info", ("mroonga create comment string"));
+  if (
+    !(param_string = my_strndup(field->comment.str,
+                                field->comment.length,
+                                MYF(MY_WME)))
+  ) {
+    error = HA_ERR_OUT_OF_MEM;
+    goto error_alloc_param_string;
+  }
+  DBUG_PRINT("info", ("mroonga comment string=%s", param_string));
+
+  sprit_ptr[0] = param_string;
+  while (sprit_ptr[0])
+  {
+    if ((sprit_ptr[1] = strchr(sprit_ptr[0], ',')))
+    {
+      *sprit_ptr[1] = '\0';
+      sprit_ptr[1]++;
+    }
+    tmp_ptr = sprit_ptr[0];
+    sprit_ptr[0] = sprit_ptr[1];
+    while (*tmp_ptr == ' ' || *tmp_ptr == '\r' ||
+      *tmp_ptr == '\n' || *tmp_ptr == '\t')
+      tmp_ptr++;
+
+    if (*tmp_ptr == '\0')
+      continue;
+
+    title_length = 0;
+    start_ptr = tmp_ptr;
+    while (*start_ptr != ' ' && *start_ptr != '\'' &&
+      *start_ptr != '"' && *start_ptr != '\0' &&
+      *start_ptr != '\r' && *start_ptr != '\n' &&
+      *start_ptr != '\t')
+    {
+      title_length++;
+      start_ptr++;
+    }
+
+    switch (title_length)
+    {
+      case 0:
+        continue;
+      case 4:
+        MRN_PARAM_STR_LIST("type", col_type, i);
+        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
+        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
+          MYF(0), tmp_ptr);
+        goto error;
+      case 5:
+        MRN_PARAM_STR_LIST("flags", col_flags, i);
+        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
+        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
+          MYF(0), tmp_ptr);
+        goto error;
+      default:
+        error = ER_MRN_INVALID_TABLE_PARAM_NUM;
+        my_printf_error(error, ER_MRN_INVALID_TABLE_PARAM_STR,
+          MYF(0), tmp_ptr);
+        goto error;
+    }
+  }
+
+  if (param_string)
+    my_free(param_string, MYF(0));
+  DBUG_RETURN(0);
+
+error:
+  if (param_string)
+    my_free(param_string, MYF(0));
+error_alloc_param_string:
+  DBUG_RETURN(error);
+}
+
+int mrn_parse_column_param(MRN_SHARE *share, TABLE *table)
+{
+  int error;
+  MRN_DBUG_ENTER_FUNCTION();
+  for (uint i = 0; i < table->s->fields; i++)
+  {
+    Field *field = table->s->field[i];
+
+    if (LEX_STRING_IS_EMPTY(field->comment)) {
+      continue;
+    }
+
+    if ((error = mrn_add_column_param(share, field, i)))
+      goto error;
+  }
+  DBUG_RETURN(0);
+
+error:
+  DBUG_RETURN(error);
+}
+
 int mrn_free_share_alloc(
   MRN_SHARE *share
 ) {
   uint i;
-  DBUG_ENTER("mrn_free_share_alloc");
+  MRN_DBUG_ENTER_FUNCTION();
   if (share->engine)
     my_free(share->engine, MYF(0));
   for (i = 0; i < share->table_share->keys; i++)
   {
+    if (share->index_table && share->index_table[i])
+      my_free(share->index_table[i], MYF(0));
     if (share->key_parser[i])
       my_free(share->key_parser[i], MYF(0));
   }
+  for (i = 0; i < share->table_share->fields; i++)
+  {
+    if (share->col_flags && share->col_flags[i])
+      my_free(share->col_flags[i], MYF(0));
+    if (share->col_type && share->col_type[i])
+      my_free(share->col_type[i], MYF(0));
+  }
   DBUG_RETURN(0);
 }
 
 MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
 {
   MRN_SHARE *share;
-  char *tmp_name, **key_parser;
-  uint length, *wrap_key_nr, *key_parser_length, i, j;
+  char *tmp_name, **index_table, **key_parser, **col_flags, **col_type;
+  uint length, *wrap_key_nr, *index_table_length;
+  uint *key_parser_length, *col_flags_length, *col_type_length, i, j;
   KEY *wrap_key_info;
   TABLE_SHARE *wrap_table_share;
-  DBUG_ENTER("mrn_get_share");
+  MRN_DBUG_ENTER_FUNCTION();
   length = (uint) strlen(table_name);
   pthread_mutex_lock(&mrn_open_tables_mutex);
   if (!(share = (MRN_SHARE*) my_hash_search(&mrn_open_tables,
@@ -651,8 +757,14 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
       my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
         &share, sizeof(*share),
         &tmp_name, length + 1,
+        &index_table, sizeof(char *) * table->s->keys,
+        &index_table_length, sizeof(uint) * table->s->keys,
         &key_parser, sizeof(char *) * table->s->keys,
         &key_parser_length, sizeof(uint) * table->s->keys,
+        &col_flags, sizeof(char *) * table->s->fields,
+        &col_flags_length, sizeof(uint) * table->s->fields,
+        &col_type, sizeof(char *) * table->s->fields,
+        &col_type_length, sizeof(uint) * table->s->fields,
         &wrap_key_nr, sizeof(*wrap_key_nr) * table->s->keys,
         &wrap_key_info, sizeof(*wrap_key_info) * table->s->keys,
         &wrap_table_share, sizeof(*wrap_table_share),
@@ -664,13 +776,20 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
     share->use_count = 0;
     share->table_name_length = length;
     share->table_name = tmp_name;
+    share->index_table = index_table;
+    share->index_table_length = index_table_length;
     share->key_parser = key_parser;
     share->key_parser_length = key_parser_length;
+    share->col_flags = col_flags;
+    share->col_flags_length = col_flags_length;
+    share->col_type = col_type;
+    share->col_type_length = col_type_length;
     strmov(share->table_name, table_name);
     share->table_share = table->s;
 
     if (
       (*error = mrn_parse_table_param(share, table)) ||
+      (*error = mrn_parse_column_param(share, table)) ||
       (*error = mrn_parse_index_param(share, table))
     )
       goto error_parse_table_param;
@@ -746,7 +865,7 @@ error_alloc_share:
 
 int mrn_free_share(MRN_SHARE *share)
 {
-  DBUG_ENTER("mrn_free_share");
+  MRN_DBUG_ENTER_FUNCTION();
   pthread_mutex_lock(&mrn_open_tables_mutex);
   if (!--share->use_count)
   {
@@ -770,7 +889,7 @@ TABLE_SHARE *mrn_get_table_share(TABLE_LIST *table_list, int *error)
   my_hash_value_type hash_value;
 #endif
   THD *thd = current_thd;
-  DBUG_ENTER("mrn_get_table_share");
+  MRN_DBUG_ENTER_FUNCTION();
 #if MYSQL_VERSION_ID >= 50603 && !defined(MRN_MARIADB_P)
   const char *key;
   key_length = get_table_def_key(table_list, &key);
@@ -835,7 +954,7 @@ KEY *mrn_create_key_info_for_table(MRN_SHARE *share, TABLE *table, int *error)
 {
   uint *wrap_key_nr = share->wrap_key_nr, i, j;
   KEY *wrap_key_info;
-  DBUG_ENTER("mrn_create_key_info_for_table");
+  MRN_DBUG_ENTER_FUNCTION();
   if (share->wrap_keys)
   {
     if (!(wrap_key_info = (KEY *)
@@ -864,7 +983,7 @@ KEY *mrn_create_key_info_for_table(MRN_SHARE *share, TABLE *table, int *error)
 void mrn_set_bitmap_by_key(MY_BITMAP *map, KEY *key_info)
 {
   uint i;
-  DBUG_ENTER("mrn_set_bitmap_by_key");
+  MRN_DBUG_ENTER_FUNCTION();
   for (i = 0; i < KEY_N_KEY_PARTS(key_info); i++)
   {
     Field *field = key_info->key_part[i].field;
@@ -875,7 +994,7 @@ void mrn_set_bitmap_by_key(MY_BITMAP *map, KEY *key_info)
 
 st_mrn_slot_data *mrn_get_slot_data(THD *thd, bool can_create)
 {
-  DBUG_ENTER("mrn_get_slot_data");
+  MRN_DBUG_ENTER_FUNCTION();
   st_mrn_slot_data *slot_data =
     (st_mrn_slot_data*) *thd_ha_data(thd, mrn_hton_ptr);
   if (slot_data == NULL) {
@@ -901,7 +1020,7 @@ st_mrn_slot_data *mrn_get_slot_data(THD *thd, bool can_create)
 
 void mrn_clear_alter_share(THD *thd)
 {
-  DBUG_ENTER("mrn_clear_alter_share");
+  MRN_DBUG_ENTER_FUNCTION();
   st_mrn_slot_data *slot_data = mrn_get_slot_data(thd, FALSE);
   if (slot_data) {
     if (slot_data->first_alter_share) {
