@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2011 Kentoku SHIBA
-  Copyright(C) 2011 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2011-2013 Kentoku SHIBA
+  Copyright(C) 2011-2013 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -39,10 +39,18 @@ typedef struct st_mroonga_share
 
   char               *engine;
   int                engine_length;
+  char               *default_tokenizer;
+  int                default_tokenizer_length;
   plugin_ref         plugin;
   handlerton         *hton;
+  char               **index_table;
   char               **key_parser;
+  char               **col_flags;
+  char               **col_type;
+  uint               *index_table_length;
   uint               *key_parser_length;
+  uint               *col_flags_length;
+  uint               *col_type_length;
   uint               *wrap_key_nr;
   uint               wrap_keys;
   uint               base_keys;
@@ -118,8 +126,6 @@ struct st_mrn_slot_data
   table->key_info = file->base_key_info; \
   table->s = share->table_share;
 
-char *mrn_create_string(const char *str, uint length);
-char *mrn_get_string_between_quote(char *ptr, bool alloc);
 #ifdef WITH_PARTITION_STORAGE_ENGINE
 void mrn_get_partition_info(const char *table_name, uint table_name_length,
                             const TABLE *table, partition_element **part_elem,
@@ -129,6 +135,8 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table);
 bool mrn_is_geo_key(const KEY *key_info);
 int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i);
 int mrn_parse_index_param(MRN_SHARE *share, TABLE *table);
+int mrn_add_column_param(MRN_SHARE *share, Field *field, int i);
+int mrn_parse_column_param(MRN_SHARE *share, TABLE *table);
 MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error);
 int mrn_free_share_alloc(MRN_SHARE *share);
 int mrn_free_share(MRN_SHARE *share);
