@@ -1,4 +1,22 @@
 #!/bin/sh
+#
+# Copyright(C) 2010  Tetsuro IKEDA
+# Copyright(C) 2010-2013  Kouhei Sutou <kou@clear-code.com>
+# Copyright(C) 2011  Kazuhiko
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 2.1 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 export BASE_DIR="$(cd $(dirname $0); pwd)"
 top_dir="$BASE_DIR/.."
@@ -144,23 +162,40 @@ if [ -n "${plugins_dir}" ]; then
 fi
 
 test_suite_names=""
+test_names=""
 while [ $# -gt 0 ]; do
     case "$1" in
 	--*)
 	    break
 	    ;;
 	*)
-	    if [ -d "$1" ]; then
-		test_suite_name=$(cd "$1" && pwd)
-	    else
-		test_suite_name="$1"
-	    fi
+	    case "$1" in
+		*/t/*.test)
+		    test_suite_name=$(echo "$1" | sed -e 's,/t/.*\.test,,g')
+		    test_suite_name=$(cd "$test_suite_name" && pwd)
+		    test_name=$(echo "$1" | sed -e 's,.*/t/\(.*\)\.test,\1,g')
+		    ;;
+		*)
+		    if [ -d "$1" ]; then
+			test_suite_name=$(cd "$1" && pwd)
+		    else
+			test_suite_name="$1"
+		    fi
+		    test_name=""
+		    ;;
+	    esac
 	    shift
-	    test_suite_name=$(echo "$test_suite_name" | sed -e "s,^${suite_dir},,")
+	    test_suite_name=$(echo "$test_suite_name" | sed -e "s,^${suite_dir}/,,")
 	    if [ -n "${test_suite_names}" ]; then
 		test_suite_names="${test_suite_names},"
 	    fi
 	    test_suite_names="${test_suite_names}${test_suite_name}"
+	    if [ -n "${test_name}" ]; then
+		if [ -n "${test_names}" ]; then
+		    test_names="${test_names},"
+		fi
+		test_names="${test_names}${test_name}"
+	    fi
 	    ;;
     esac
 done
@@ -169,11 +204,17 @@ if [ -z "$test_suite_names" ]; then
     test_suite_names="${all_test_suite_names}"
 fi
 
+mysql_test_run_args=""
+mysql_test_run_args="${mysql_test_run_args} --no-check-testcases"
+mysql_test_run_args="${mysql_test_run_args} --parallel=${n_processors}"
+mysql_test_run_args="${mysql_test_run_args} --retry=1"
+mysql_test_run_args="${mysql_test_run_args} --suite=${test_suite_names}"
+mysql_test_run_args="${mysql_test_run_args} --force"
+if [ -n "$test_names" ]; then
+    mysql_test_run_args="${mysql_test_run_args} --do-test=${test_names}"
+fi
+
 (cd "$build_mysql_test_dir" && \
     ./mysql-test-run.pl \
-    --no-check-testcases \
-    --parallel="${n_processors}" \
-    --retry=1 \
-    --suite="${test_suite_names}" \
-    --force \
+    ${mysql_test_run_args} \
     "$@")
